@@ -1,4 +1,5 @@
 import sys
+import json
 
 
 def render(node, strict=False):
@@ -50,6 +51,9 @@ def render_list(node):
 
 
 def render_node(node, strict=False):
+    if node["type"] not in nodes_rendering_order:
+        raise Exception("There are no defined rules for rendering a node of type '%s', has it been defined in render.py?" % node["type"])
+
     for key_type, render_key, dependent in nodes_rendering_order[node['type']]:
         if not dependent:
             continue
@@ -63,11 +67,11 @@ def render_node(node, strict=False):
         if strict:
             try:
                 if key_type == "key":
-                    assert isinstance(node[render_key], (dict, type(None)))
+                    assert isinstance(node[render_key], (dict, type(None))), "Key '%s' is expected to have type of 'key' (dict/None) but has type of '%s' instead" % (render_key, type(node[render_key]))
                 elif key_type == "string":
-                    assert isinstance(node[render_key], str)
+                    assert isinstance(node[render_key], str), "Key '%s' is expected to have type of 'string' but has type of '%s' instead" % (render_key, type(node[render_key]))
                 elif key_type in ("list", "formatting"):
-                    assert isinstance(node[render_key], list)
+                    assert isinstance(node[render_key], list), "Key '%s' is expected to have type of 'list' but has type of '%s' instead" % (render_key, type(node[render_key]))
                 elif key_type == "constant":
                     pass
                 else:
@@ -80,7 +84,7 @@ def render_node(node, strict=False):
                 elif isinstance(dependent, list):
                     assert all([x in node for x in dependent])
             except AssertionError as e:
-                sys.stdout.write("Where node.type == '%s', render_key == '%s' and node == %s\n" % (node["type"], render_key, node))
+                sys.stdout.write("Where node.type == '%s', render_key == '%s' and node ==\n%s\n" % (node["type"], render_key, json.dumps(node, indent=4, sort_keys=True)))
                 raise e
 
         if key_type in ['key', 'string', 'list', 'formatting']:
@@ -147,6 +151,13 @@ nodes_rendering_order = {
             ("string",     "value",             True),
             ("formatting", "second_formatting", True),
         ],
+
+        "star_expression": [
+            ("constant",   "*",                 True),
+            ("formatting", "formatting",        True),
+            ("key",        "value",             True),
+        ],
+
         "string": [
             ("formatting", "first_formatting",  True),
             ("string",     "value",             True),
@@ -320,6 +331,10 @@ nodes_rendering_order = {
             ("list",       "arguments",         True),
             ("formatting", "fourth_formatting", True),
             ("constant",   ")",                 True),
+            ("formatting", "return_annotation_first_formatting", "return_annotation"),
+            ("constant",   "->", "return_annotation"),
+            ("formatting", "return_annotation_second_formatting", "return_annotation"),
+            ("key",        "return_annotation", "return_annotation"),
             ("formatting", "fifth_formatting",  True),
             ("constant",   ":",                 True),
             ("formatting", "sixth_formatting",  True),
@@ -335,6 +350,10 @@ nodes_rendering_order = {
         ],
         "def_argument": [
             ("key",        "target",            True),
+            ("formatting", "annotation_first_formatting", "annotation"),
+            ("constant",   ":",                 "annotation"),
+            ("formatting", "annotation_second_formatting", "annotation"),
+            ("key",        "annotation",        "annotation"),
             ("formatting", "first_formatting",  "value"),
             ("constant",   "=",                 "value"),
             ("formatting", "second_formatting", "value"),
@@ -344,6 +363,10 @@ nodes_rendering_order = {
             ("constant",   "*",                 True),
             ("formatting", "formatting",        True),
             ("key",        "value",             True),
+            ("formatting", "annotation_first_formatting", "annotation"),
+            ("constant",   ":",                 "annotation"),
+            ("formatting", "annotation_second_formatting", "annotation"),
+            ("key",        "annotation",        "annotation"),
         ],
         "kwargs_only_marker": [
             ("constant",   "*",                 True),
@@ -353,6 +376,10 @@ nodes_rendering_order = {
             ("constant",   "**",                True),
             ("formatting", "formatting",        True),
             ("key",        "value",             True),
+            ("formatting", "annotation_first_formatting", "annotation"),
+            ("constant",   ":",                 "annotation"),
+            ("formatting", "annotation_second_formatting", "annotation"),
+            ("key",        "annotation",        "annotation"),
         ],
 
         "return": [
@@ -366,7 +393,7 @@ nodes_rendering_order = {
             ("formatting", "first_formatting",  True),
             ("key",        "value",             "value"),
             ("formatting", "second_formatting", "instance"),
-            ("constant",   ",",                 "instance"),
+            ("string",     "comma_or_from",     "instance"),
             ("formatting", "third_formatting",  "instance"),
             ("key",        "instance",          "instance"),
             ("formatting", "fourth_formatting", "traceback"),
@@ -472,14 +499,26 @@ nodes_rendering_order = {
 
         "assignment": [
             ("key",        "target",            True),
+            ("formatting", "annotation_first_formatting",  "annotation"),
+            ("constant",   ":",                 "annotation"),
+            ("formatting", "annotation_second_formatting",  "annotation"),
+            ("key",        "annotation",        "annotation"),
             ("formatting", "first_formatting",  True),
             # FIXME should probably be a different node type
             ("string",     "operator",          "operator"),
-            ("constant",   "=",                 True),
+            ("constant",   "=",                 "target"),
             ("formatting", "second_formatting", True),
             ("key",        "value",             True),
         ],
 
+        "standalone_annotation": [
+            ("key",        "target",            True),
+            ("formatting", "first_formatting",  True),
+            ("constant",   ":",                 True),
+            ("formatting", "second_formatting", True),
+            ("key",        "annotation",        True),
+        ],
+
         "unitary_operator": [
             ("string",     "value",             True),
             ("formatting", "formatting",        True),
@@ -730,14 +769,6 @@ nodes_rendering_order = {
             ("formatting", "second_formatting", "target"),
             ("string",     "target",            "target"),
         ],
-        "typed_name": [
-            ("string",     "value",             True),
-            ("formatting", "first_formatting",  True),
-            ("constant",   ":",                 True),
-            ("formatting", "second_formatting", True),
-            ("key",        "annotation",        True),
-        ],
-
         "print": [
             ("constant",   "print",                  True),
             ("formatting", "formatting",             True),
